﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using DocumentFormat.OpenXml.Packaging;

namespace OpenXmlPowerTools.SpreadsheetWriter
{
    class Program
    {
        static void Test1()
        {
            Workbook wb = new Workbook
            {
                Worksheets = new Worksheet[]
                {
                    new Worksheet
                    {
                        Name = "MyFirstSheet",
                        TableName = "NamesAndRates",
                        ColumnHeadings = new Cell[]
                        {
                            new Cell
                            {
                                Value = "Name",
                                Bold = true,
                            },
                            new Cell
                            {
                                Value = "Age",
                                Bold = true,
                                HorizontalCellAlignment = HorizontalCellAlignment.Left,
                            },
                            new Cell
                            {
                                Value = "Rate",
                                Bold = true,
                                HorizontalCellAlignment = HorizontalCellAlignment.Left,
                            }
                        },
                        Rows = new Row[]
                        {
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "Eric",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = 50,
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = (decimal)45.00,
                                        FormatCode = "0.00",
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "Bob",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = 42,
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = (decimal)78.00,
                                        FormatCode = "0.00",
                                    },
                                }
                            },
                        }
                    }
                }
            };
            SpreadsheetWriter.Write("Test1.xlsx", wb);
        }

        static void Test2()
        {
            Workbook wb = new Workbook
            {
                Worksheets = new Worksheet[]
                {
                    new Worksheet
                    {
                        Name = "MyFirstSheet",
                        ColumnHeadings = new Cell[]
                        {
                            new Cell
                            {
                                Value = "DataType",
                                Bold = true,
                            },
                            new Cell
                            {
                                Value = "Value",
                                Bold = true,
                                HorizontalCellAlignment = HorizontalCellAlignment.Right,
                            },
                        },
                        Rows = new Row[]
                        {
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "Boolean",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Boolean,
                                        Value = true,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "Boolean",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Boolean,
                                        Value = false,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "String",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "A String",
                                        HorizontalCellAlignment = HorizontalCellAlignment.Right,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "int",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = (int)100,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "int?",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = (int?)100,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "int? (is null)",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = null,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "uint",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = (uint)101,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "long",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = Int64.MaxValue,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "float",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = (float)123.45,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "double",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = (double)123.45,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.String,
                                        Value = "decimal",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = (decimal)123.45,
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.Date,
                                        Value = new DateTime(2012, 1, 8),
                                        FormatCode = "mm-dd-yy",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Date,
                                        Value = new DateTime(2012, 1, 9),
                                        FormatCode = "mm-dd-yy",
                                        Bold = true,
                                        HorizontalCellAlignment = HorizontalCellAlignment.Center,
                                    },
                                }
                            },
                        }
                    }
                }
            };
            SpreadsheetWriter.Write("Test2.xlsx", wb);
        }

        static void Test3()
        {
            string[] ColumnHeadings = new[] {
                "Abc",
                "Def",
                "Ghi",
                "Jkl",
                "Mno",
                "Pqr",
                "Stu",
                "Vwx",
                "Aaa",
                "Bbb",
                "Ccc",
                "Ddd",
                "Eee",
                "Fff",
                "Ggg",
                "Hhh",
                "Iii",
                "Jjj",
            };

            var data = new [] {
                new {
                    SheetName = "HugeSheet1",
                    NumberColumns = 8,
                    NumberRows = 300000,
                },
                new {
                    SheetName = "MyHugeSheet2",
                    NumberColumns = 12,
                    NumberRows = 2000,
                },
                new {
                    SheetName = "SmallSheet",
                    NumberColumns = 10,
                    NumberRows = 20,
                },
            };

            Random rand = new Random();
            Workbook wb = new Workbook
            {
                Worksheets = data.Select(d => new Worksheet
                    {
                        Name = d.SheetName,
                        ColumnHeadings = Enumerable.Range(0, d.NumberColumns)
                            .Select(c => new Cell
                                {
                                    Value = ColumnHeadings[c],
                                    Bold = true,
                                }),
                        Rows = Enumerable.Range(0, d.NumberRows)
                            .Select(r => new Row
                            {
                                Cells = Enumerable.Range(0, d.NumberColumns)
                                .Select(c => 
                                    {
                                        switch (rand.Next(1, 5))
                                        {
                                            case 1:
                                                return new Cell
                                                {
                                                    Value = ((char)((int)'A' + rand.Next(0, 25))).ToString() +
                                                        ((char)((int)'A' + rand.Next(0, 25))).ToString() +
                                                        ((char)((int)'A' + rand.Next(0, 25))).ToString(),
                                                    CellDataType = CellDataType.String,
                                                };
                                            case 2:
                                                return new Cell
                                                {
                                                    Value = rand.NextDouble() * 100,
                                                    CellDataType = CellDataType.Number,
                                                    FormatCode = "0.00",
                                                };
                                            case 3:
                                                return new Cell
                                                {
                                                    CellDataType = CellDataType.Date,
                                                    Value = (new DateTime(2012, 1, 8)).AddDays(rand.Next(0, 1000)),
                                                    FormatCode = "mm-dd-yy",
                                                };
                                            case 4:
                                                return new Cell
                                                {
                                                    CellDataType = CellDataType.Boolean,
                                                    Value = rand.Next(1,2) == 1 ? true : false,
                                                };
                                            case 5:
                                                return new Cell
                                                {
                                                    Value = rand.NextDouble() * 100,
                                                    CellDataType = CellDataType.Number,
                                                    FormatCode = "0.0000",
                                                };
                                            case 6:
                                                return new Cell
                                                {
                                                    Value = ((char)((int)'A' + rand.Next(0, 25))).ToString() +
                                                        ((char)((int)'A' + rand.Next(0, 25))).ToString() +
                                                        ((char)((int)'A' + rand.Next(0, 25))).ToString(),
                                                    CellDataType = CellDataType.String,
                                                    HorizontalCellAlignment = HorizontalCellAlignment.Right,
                                                };
                                        }
                                        return (Cell)null;
                                    })
                            }),
                    })
            };
            SpreadsheetWriter.Write("Test3.xlsx", wb);
        }

        static void Test4()
        {
            Workbook wb = new Workbook
            {
                Worksheets = new Worksheet[]
                {
                    new Worksheet
                    {
                        Name = "MyFirstSheet",
                        TableName = "TheTable",
                        ColumnHeadings = new Cell[]
                        {
                            new Cell
                            {
                                Value = "Nbr",
                                Bold = true,
                            },
                            new Cell
                            {
                                Value = "Name",
                                Bold = true,
                            },
                            new Cell
                            {
                                Value = "Age",
                                Bold = true,
                            }
                        },
                        Rows = new Row[]
                        {
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.Date,
                                        Value = new DateTime(2012, 1, 8),
                                        FormatCode = "mm-dd-yy",
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Date,
                                        Value = new DateTime(2012, 1, 9),
                                        FormatCode = "mm-dd-yy",
                                        Bold = true,
                                        HorizontalCellAlignment = HorizontalCellAlignment.Center,
                                    },
                                    new Cell { CellDataType = CellDataType.String, Value = "def" },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = 123,
                                        FormatCode = "0.000000",
                                        Bold = true,
                                        HorizontalCellAlignment = HorizontalCellAlignment.Left,
                                        Italic = true,
                                    },
                                    new Cell {
                                        CellDataType = CellDataType.Number,
                                        Value = 456,
                                        FormatCode = "0.000"
                                    },
                                    new Cell { CellDataType = CellDataType.Number,
                                        Value = 789,
                                        FormatCode = "0.0"
                                    },
                                }
                            },
                            new Row
                            {
                                Cells = new Cell[]
                                {
                                    new Cell { CellDataType = CellDataType.String, Value = "abc" },
                                    new Cell { CellDataType = CellDataType.String, Value = "def" },
                                    new Cell { CellDataType = CellDataType.String, Value = "ghi" },
                                }
                            },
                        }
                    }
                }
            };
            SpreadsheetWriter.Write("Test4.xlsx", wb);
        }

        static void Main(string[] args)
        {
            Test1();
            //Test2();
            //Test3();
            //Test4();
        }
    }
}
